OwnedJsValue

Struct OwnedJsValue 

pub struct OwnedJsValue { /* private fields */ }
OwnedJsValue wraps a Javascript value owned by the QuickJs runtime.

Guarantees cleanup of resources by dropping the value from the runtime.

Safety:

This type is Send and Sync only for convenience, since OwnedJsValue itself is just a wrapper around a raw pointer. But any operation on the underlying raw pointer is unsafe. Make sure using it in a same thread.

Implementations

Source

impl OwnedJsValue

pub fn context(&self) -> *mut JSContext

pub fn new(context: *mut JSContext, value: JSValue) -> Self

Create a new OwnedJsValue from a JsValue. This will NOT increase the ref count of the underlying value. So you have to manage memory yourself. Be careful when using this.

pub fn own(context: *mut JSContext, value: &JSValue) -> Self

Create a new OwnedJsValue from a JsValue. This will increase the ref count of the underlying value.

pub fn tag(&self) -> JsTag

pub unsafe fn as_inner(&self) -> &JSValue

Get the inner JSValue without increasing ref count.

Unsafe because the caller must ensure proper memory management.

pub unsafe fn extract(self) -> JSValue

Extract the underlying JSValue.

Unsafe because the caller must ensure memory management. (eg JS_FreeValue)

pub fn replace(&mut self, new: JSValue)

Replace the underlying JSValue. This will decrease the ref count of the old value but remain the ref count of the new value.

pub fn is_null(&self) -> bool

Check if this value is null.

pub fn is_undefined(&self) -> bool

Check if this value is undefined.

pub fn is_bool(&self) -> bool

Check if this value is bool.

pub fn is_int(&self) -> bool

Check if this value is int.

pub fn is_bigint(&self) -> bool

Check if this value is BigInt.

pub fn is_short_bigint(&self) -> bool

Check if this value is BigInt, but short enough to fit in a i32

pub fn is_float(&self) -> bool

Check if this value is float.

pub fn is_exception(&self) -> bool

Check if this value is a Javascript exception.

pub fn is_object(&self) -> bool

Check if this value is a Javascript object.

pub fn is_array(&self) -> bool

Check if this value is a Javascript array.

pub fn is_array_buffer(&self) -> bool

Check if this value is a Javascript array.

pub fn is_proxy(&self) -> bool

Check if this value is a Javascript Proxy object.

pub fn is_function(&self) -> bool

Check if this value is a Javascript function.

pub fn is_constructor(&self) -> bool

Check if this value is a Javascript function.

pub fn is_promise(&self) -> bool

Check if this value is a Javascript promise.

pub fn is_error(&self) -> bool

Check if this value is a Javascript error.

pub fn is_regexp(&self) -> bool

Check if this value is a Javascript RegExp.

pub fn is_set(&self) -> bool

Check if this value is a Javascript Set.

pub fn is_map(&self) -> bool

Check if this value is a Javascript Map.

pub fn is_weak_set(&self) -> bool

Check if this value is a Javascript WeakSet.

pub fn is_weak_map(&self) -> bool

Check if this value is a Javascript WeakMap.

pub fn is_weak_ref(&self) -> bool

Check if this value is a Javascript WeakRef.

pub fn is_data_view(&self) -> bool

Check if this value is a Javascript DataView.

pub fn is_module(&self) -> bool

Check if this value is a Javascript module.

pub fn is_string(&self) -> bool

Check if this value is a Javascript string.

pub fn is_compiled_function(&self) -> bool

Check if this value is a bytecode compiled function.

pub fn to_bool(&self) -> Result<bool, ValueError>

Convert this value into a bool

pub fn to_int(&self) -> Result<i32, ValueError>

Convert this value into an i32

pub fn to_float(&self) -> Result<f64, ValueError>

Convert this value into an f64

pub fn to_string(&self) -> Result<String, ValueError>

Convert this value into a string

pub fn to_array(&self) -> Result<OwnedJsArray, ValueError>

pub fn get_proxy_target( &self, recursive: bool, ) -> Result<OwnedJsValue, ValueError>

pub fn try_into_object(self) -> Result<OwnedJsObject, ValueError>

Try convert this value into a object

pub fn to_date(&self) -> Result<DateTime<Utc>, ValueError>

pub fn to_bigint(&self) -> Result<BigInt, ValueError>

pub fn try_into_function(self) -> Result<JsFunction, ValueError>

Try convert this value into a function

pub fn try_into_promise(self) -> Result<OwnedJsPromise, ValueError>

Try convert this value into a function

pub fn try_into_compiled_function( self, ) -> Result<JsCompiledFunction, ValueError>

Try convert this value into a compiled function

pub fn try_into_module(self) -> Result<JsModule, ValueError>

Try convert this value into a module

pub fn js_to_string(&self) -> Result<String, ExecutionError>

Call the Javascript .toString() method on this value.

pub fn to_json_string(&self, space: u8) -> Result<String, ExecutionError>

Call the Javascript JSON.stringify() method on this value.